from .tools import MttsLangModel
from aiohttp import ClientSession
from .tools import get_connect_id, TOKEN_URL, LANG_MODEL_URL, FIRST_JSON, SECOND_JSON, THIRD_SSML, WSS_CONNECT_URL
from .tools import get_time
import re
from .tools import MttsException
import json
import websockets as ws
from websockets import ConnectionClosed
from typing import List


class Mtts:
    connection_id: str = get_connect_id()
    token: str = None
    _token_url: str = TOKEN_URL
    _lang_model_url: str = LANG_MODEL_URL
    lang_models: List[MttsLangModel] = []
    SpeechSDK_VERSION: str = "1.19.0"

    async def get_token(self) -> None:
        async with ClientSession() as session:
            async with session.get(self._token_url) as response:
                response = await response.read()
                to = re.compile(r"token: \"(.*?)\"")
                token = to.search(response.decode("utf-8")).group()
                if token is None:
                    raise MttsException("not found token")
                self.token = token.split("token: ")[1].replace('"', '')

    def token_check(func):
        async def wrapper(self, *args, **kwargs):
            if self.token is None:
                await self.get_token()
            return await func(self, *args, **kwargs)

        return wrapper

    def model_check(func):
        async def wrapper(self, *args, **kwargs):
            if len(self.lang_models) == 0:
                await self.get_lang_models()
            return await func(self, *args, **kwargs)

        return wrapper

    @token_check
    async def get_lang_models(self) -> list:
        '''
         get all support voice models
        :return: list
        '''
        if len(self.lang_models) == 0:
            headers = {"authorization": 'Bearer {}'.format(self.token)}
            async with ClientSession(headers=headers) as session:
                async with session.get(self._lang_model_url) as response:
                    response = await response.read()
                    data = json.loads(response)
                    self.lang_models.extend([MttsLangModel(m) for m in data])

        return self.lang_models

    @token_check
    async def mtts(self, text: str, short_name: str, style: str = "general", rate: int = 0, pitch: int = 0,
                   kmhz: int = 24) -> bytes:
        '''
        :param text: text to convert
        :param short_name:The name of the language pack to use
        :param style:The language style to use, defaults to 'general'
        :param rate:Speech rate, default 0
        :param pitch: intonation, default 0
        :param kmhz:
        :return: bytes,buffer of audio
        '''
        async with ws.connect(WSS_CONNECT_URL.format(self.token, self.connection_id)) as websocket:

            await websocket.send(FIRST_JSON.format(self.connection_id, get_time, self.SpeechSDK_VERSION))
            await websocket.send(SECOND_JSON.format(self.connection_id, get_time(), kmhz))
            await websocket.send(
                THIRD_SSML.format(self.connection_id, get_time(), short_name, style, rate, pitch, text))
            await websocket.recv()
            await websocket.recv()
            await websocket.recv()
            final_date = b''
            while True:
                try:
                    data = await websocket.recv()
                    if type(data) is bytes:
                        if b'Path:audio' in data:
                            final_date += data[data.index(b"Path:audio") + 12:]
                    elif "Path:turn.end" in data:
                        return final_date
                except ConnectionClosed as e:
                    if e.code == 1006:
                        await asyncio.sleep(2)
                        break



import setuptools
import pymtts

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
requires_list = open('./requirements.txt', 'r', encoding='utf8').readlines()
requires_list = [i.strip() for i in requires_list]
setuptools.setup(
    name=pymtts.name,
    version=pymtts.VERSION,
    author="p1ay8y3ar",
    author_email="p1ay8y3ar@gmail.com",
    description="A python package for using Azure smart AI speech",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/p1ay8y3ar/pymtts",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=requires_list
)



class MttsLangModel:
    Name = None
    DisplayName = None
    LocalName = None
    ShortName = None
    Gender = None
    Locale = None
    LocaleName = None
    StyleList = []
    SampleRateHertz = None
    VoiceType = None
    Status = None
    RolePlayList = []

    def __init__(self, arg: dict):
        for key, value in arg.items():
            setattr(self, key, value)

    def keys(self):
        return (
            "Name", "DisplayName", "LocalName", "ShortName", "Gender",
            "Locale", "LocaleName", "StyleList", "SampleRateHertz", "VoiceType",
            "Status", "RolePlayList")

    def __getitem__(self, item):
        return getattr(self, item)


class MttsException(Exception):
    pass

from datetime import datetime
import uuid


def get_time():
    now = datetime.now()
    return now.strftime("%Y-%m-%dT%H:%M:%SZ")


def get_connect_id():
    return str(uuid.uuid4()).replace("-", "").upper()


TOKEN_URL: str = "https://azure.microsoft.com/en-gb/services/cognitive-services/text-to-speech/"
LANG_MODEL_URL: str = "https://eastus.tts.speech.microsoft.com/cognitiveservices/voices/list"
WSS_CONNECT_URL: str = "wss://eastus.tts.speech.microsoft.com/cognitiveservices/websocket/v1?Authorization=bearer%20{}&X-ConnectionId={}"
FIRST_JSON: str = '''Path: speech.config\r\nX-RequestId: {}\r\nX-Timestamp: {}\r\nContent-Type: application/json\r\n\r\n{{"context": {{"system": {{"name": "SpeechSDK","version": "{}","build": "JavaScript","lang": "JavaScript"}},"os": {{"platform": "Browser/MacIntel","name": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.0.0 Safari/537.36","version": "5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.0.0 Safari/537.36"}}}}}}'''

SECOND_JSON: str = '''Path: synthesis.context\r\nX-RequestId: {}\r\nX-Timestamp: {}\r\nContent-Type: application/json\r\n\r\n{{"synthesis": {{"audio": {{"metadataOptions": {{"bookmarkEnabled": false,"sentenceBoundaryEnabled": false,"visemeEnabled": false,"wordBoundaryEnabled": false}},"outputFormat": "audio-{}khz-160kbitrate-mono-mp3"}},"language": {{"autoDetection": false}}}}}}'''

THIRD_SSML: str = """Path: ssml\r\nX-RequestId: {}\r\nX-Timestamp: {}\r\nContent-Type: application/ssml+xml\r\n\r\n<speak xmlns="http://www.w3.org/2001/10/synthesis" xmlns:mstts="http://www.w3.org/2001/mstts" xmlns:emo="http://www.w3.org/2009/10/emotionml" version="1.0" xml:lang="en-US"><voice name="{}"><mstts:express-as style="{}"><prosody rate="{}%" pitch="{}%">{}</prosody></mstts:express-as></voice></speak>"""
